package sun.java2d.pipe;

import java.awt.font.GlyphVector;
import sun.java2d.loops.FontInfo;
import sun.java2d.SurfaceData;
import java.awt.font.TextLayout;
import sun.font.GlyphList;
import sun.java2d.SunGraphics2D;

public abstract class GlyphListPipe implements TextPipe
{
    public GlyphListPipe() {
    }
    
    @Override
    public void drawString(SunGraphics2D g2, String s, double n, double n2) {
        FontInfo fontInfo = g2.getFontInfo();
        if (fontInfo.pixelHeight > 100) {
            SurfaceData.outlineTextRenderer.drawString(g2, s, n, n2);
            return;
        }
        float x;
        float y;
        if (g2.transformState >= 3) {
            double[] array = { n + (double)fontInfo.originX, n2 + (double)fontInfo.originY };
            g2.transform.transform(array, 0, array, 0, 1);
            x = (float)array[0];
            y = (float)array[1];
        }
        else {
            x = (float)(n + (double)fontInfo.originX + (double)g2.transX);
            y = (float)(n2 + (double)fontInfo.originY + (double)g2.transY);
        }
        GlyphList instance = GlyphList.getInstance();
        if (instance.setFromString(fontInfo, s, x, y)) {
            this.drawGlyphList(g2, instance);
            instance.dispose();
        }
        else {
            instance.dispose();
            new TextLayout(s, g2.getFont(), g2.getFontRenderContext()).draw(g2, (float)n, (float)n2);
        }
        return;
    }
    
    @Override
    public void drawChars(SunGraphics2D g2, char[] array, int n, int n2, int n3, int n4) {
        FontInfo fontInfo = g2.getFontInfo();
        if (fontInfo.pixelHeight > 100) {
            SurfaceData.outlineTextRenderer.drawChars(g2, array, n, n2, n3, n4);
            return;
        }
        float x;
        float y;
        if (g2.transformState >= 3) {
            double[] array2 = { (double)((float)n3 + fontInfo.originX), (double)((float)n4 + fontInfo.originY) };
            g2.transform.transform(array2, 0, array2, 0, 1);
            x = (float)array2[0];
            y = (float)array2[1];
        }
        else {
            x = (float)n3 + fontInfo.originX + (float)g2.transX;
            y = (float)n4 + fontInfo.originY + (float)g2.transY;
        }
        GlyphList instance = GlyphList.getInstance();
        if (instance.setFromChars(fontInfo, array, n, n2, x, y)) {
            this.drawGlyphList(g2, instance);
            instance.dispose();
        }
        else {
            instance.dispose();
            new TextLayout(new String(array, n, n2), g2.getFont(), g2.getFontRenderContext()).draw(g2, (float)n3, (float)n4);
        }
        return;
    }
    
    @Override
    public void drawGlyphVector(SunGraphics2D sunGraphics2D, GlyphVector gv, float x, float y) {
        FontInfo gvFontInfo = sunGraphics2D.getGVFontInfo(gv.getFont(), gv.getFontRenderContext());
        if (gvFontInfo.pixelHeight > 100) {
            SurfaceData.outlineTextRenderer.drawGlyphVector(sunGraphics2D, gv, x, y);
            return;
        }
        if (sunGraphics2D.transformState >= 3) {
            double[] array = { (double)x, (double)y };
            sunGraphics2D.transform.transform(array, 0, array, 0, 1);
            x = (float)array[0];
            y = (float)array[1];
        }
        else {
            x += (float)sunGraphics2D.transX;
            y += (float)sunGraphics2D.transY;
        }
        GlyphList instance = GlyphList.getInstance();
        instance.setFromGlyphVector(gvFontInfo, gv, x, y);
        this.drawGlyphList(sunGraphics2D, instance, gvFontInfo.aaHint);
        instance.dispose();
        return;
    }
    
    protected abstract void drawGlyphList(SunGraphics2D p0, GlyphList p1);
    
    protected void drawGlyphList(SunGraphics2D sunGraphics2D, GlyphList list, int n) {
        this.drawGlyphList(sunGraphics2D, list);
    }
}
